const std = @import("std");
const sdl = @import("sdl");

const Self = @This();

texture: sdl.Texture,
width: usize,
height: usize,
access: sdl.Texture.Access,
format: sdl.Texture.Format,

pub fn load(ren: sdl.Renderer, path: [:0]const u8) !Self {
    var surf = try sdl.loadBmp(path);
    defer surf.destroy();

    var tex = try sdl.createTextureFromSurface(ren, surf);
    errdefer tex.destroy();

    try tex.setBlendMode(.blend);

    const info = try tex.query();

    return .{
        .texture = tex,
        .access = info.access,
        .format = info.format,
        .width = info.width,
        .height = info.height,
    };
}

pub fn deinit(self: Self) void {
    self.texture.destroy();
}
