const std = @import("std");

pub fn Grid(T: type) type {
    return struct {
        data: [][]T,
        allocator: std.mem.Allocator,

        const Self = @This();

        pub fn init(allocator: std.mem.Allocator, w: usize, h: usize) !Self {
            const data = try allocator.alloc([]T, h);
            errdefer allocator.free(data);
            for (data) |*e| {
                e.* = try allocator.alloc(T, w);
                errdefer allocator.free(e.*);
            }
            return .{
                .data = data,
                .allocator = allocator,
            };
        }

        pub fn deinit(self: Self) void {
            for (self.data) |e|
                self.allocator.free(e);
            self.allocator.free(self.data);
        }

        pub fn width(self: Self) usize {
            return self.data[0].len;
        }

        pub fn height(self: Self) usize {
            return self.data.len;
        }

        pub fn isSquare(self: Self) bool {
            return self.width() == self.height();
        }

        pub fn at(self: Self, x: usize, y: usize) ?*T {
            if (x >= self.width() or y >= self.height())
                return null;
            return &self.data[@intCast(y)][@intCast(x)];
        }

        pub fn fill(self: Self, v: T) void {
            for (0..self.height()) |y| {
                for (0..self.width()) |x|
                    self.at(x, y).?.* = v;
            }
        }
    };
}
