const std = @import("std");
const sdl = @import("sdl");
const cfg = @import("cfg.zig");
const PixelBuffer = @import("PixelBuffer.zig");
const R = @import("R.zig");

pub fn main() !void {
    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer if (gpa.deinit() == .leak) std.log.err("GPA leak", .{});
    const allocator = gpa.allocator();

    var r = try R.init(allocator, .{
        .title = "abs/tract",
        .width = cfg.dwidth,
        .height = cfg.dheight,
        .scale = 2,
        .fps = cfg.fps,
        .chdir = false,
        .windowDim = .default,
    });
    defer r.deinit();

    var pix = try PixelBuffer.init(r.renderer, allocator, null, null);
    defer pix.deinit();
    pix.fill(.off);

    var square = try PixelBuffer.init(r.renderer, allocator, 16, 16);
    defer square.deinit();
    square.fill(.on);

    var p: usize = 1;
    while (!r.shouldQuit) {
        r.cycleEvents();
        if (r.scstates[sdl.c.SDL_SCANCODE_ESCAPE] == .press)
            r.shouldQuit = true;
        if (r.scstates[sdl.c.SDL_SCANCODE_SPACE] == .press)
            r.tick = 0;
        if (r.scstates[sdl.c.SDL_SCANCODE_BACKSPACE] == .press)
            pix.fill(.invert);
        if (r.scstates[sdl.c.SDL_SCANCODE_R] == .press)
            pix.fill(.off);
        if (r.scstates[sdl.c.SDL_SCANCODE_F] == .press)
            r.toggleFullscreen();
        if (r.scstates[sdl.c.SDL_SCANCODE_LEFT] == .press) {
            if (p > 1) p -= 2;
        }
        if (r.scstates[sdl.c.SDL_SCANCODE_RIGHT] == .press)
            p += 2;
        if (r.scstates[sdl.c.SDL_SCANCODE_LEFTBRACKET] == .press)
            pix.rotate(-1);
        if (r.scstates[sdl.c.SDL_SCANCODE_RIGHTBRACKET] == .press)
            pix.rotate(1);

        try r.drawBegin();
        try r.renderer.setColor(R.black);
        try r.renderer.clear();
        for (0..8) |i|
            sickShit(pix, square, r, 16, i, (i + 1) * 2 + p);
        try pix.present();
        try r.drawEnd();
    }
}

fn sickShit(
    pix: PixelBuffer,
    square: PixelBuffer,
    r: R,
    x: comptime_int,
    y: usize,
    p: usize,
) void {
    const lineHeight = 32;
    pix.copy(
        square,
        .invert,
        @as(i32, @intCast(r.tick)) - x,
        @intCast(r.tick % p + y * lineHeight),
    );
    pix.copy(
        square,
        .invert,
        @as(i32, @intCast(cfg.dwidth)) -
            @as(i32, @intCast(square.grid.width())) -
            @as(i32, @intCast(r.tick)) + x,
        @intCast(r.tick % p + y * lineHeight),
    );
}
